import pickle
import numpy as np
import pandas as pd
from utils import metrics
from DNP import DNP
from sklearn.model_selection import KFold, train_test_split
import warnings
warnings.filterwarnings('ignore')

def run(X, y, param):
	architecture = param["architecture"]
	reg = param["reg"]
	metrics_style = param["metrics"]
	max_features = param["max_features"]

	#10 Fold Cross Validation on the dataset
	# kf = KFold(X.shape[0], n_splits=10, shuffle=True)
	kf = KFold(n_splits=10, shuffle=True)
	cv_i = 0

	for tr_vd_id, tst_id in kf.split(X, y):
	#Among the training data, split 80% as training and 20% as validation.
		tr_id, vd_id = train_test_split(tr_vd_id, train_size=0.8)

		tr_X = X[tr_id, :]
		vd_X = X[vd_id, :]
		tst_X = X[tst_id, :]

		tr_y = y[tr_id]
		vd_y = y[vd_id]
		tst_y = y[tst_id]

		#If the training, or validation, or test data contain only one class. This cross-validation is ignored.
		if len(np.unique(tr_y)) == 1 or len(np.unique(tst_y)) == 1 or len(np.unique(vd_y)) == 1:
			continue

		cv_i += 1
		print("=" * 20 + " CV:%d " % cv_i + "=" * 20)

		dnp = DNP(hidden_depth=architecture, reg=reg, max_features=max_features, metrics_func=metrics_style)
		dnp.fit(X=tr_X, y=tr_y, tst_X=tst_X, tst_y=tst_y, vd_X=vd_X, vd_y=vd_y)


if __name__ == "__main__":

	data_name = "./data/leukemia.pkl"

	#Load the features and the labels.
	#The data should be standardized by removing the mean and scaling to unit variance.
	#The positive and negative label are represented as 1 and 0, respectively.
	X, y= pickle.load(open(data_name, 'rb'), encoding='latin1')

	param = {
		#The number of neurons in each hidden layer.
		#For binary classification problem, the number of neurons of the last layer should be 2.
		"architecture":[30, 20, 2],
		#The regularization
		"reg":0.05,
		#The dropout rate
		#Metrics including auc, mse, accuracy, and logistic loss
		"metrics": "auc",
		#The maximum number of selected features
		"max_features": 20
	}

	run(X, y, param)
